@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -22,6 +22,12 @@ WHERE CONTEXT *Context;
 
 WHERE char Errorbuf[SHORT_STRING];
 
+#if defined(DL_STANDALONE) && defined(USE_DOTLOCK)
+WHERE char *MuttDotlock;
+#endif
+
+WHERE ADDRESS *From;
+
 WHERE char *AliasFile;
 WHERE char *AliasFmt;
 WHERE char *AttachSep;
@@ -42,8 +48,10 @@ WHERE char *Homedir;
 WHERE char *Hostname;
 #ifdef USE_IMAP
 WHERE char *ImapUser INITVAL (NULL);
+WHERE char *ImapCRAMKey INITVAL (NULL);
 WHERE char *ImapPass INITVAL (NULL);
-WHERE short ImapCheckTime;
+WHERE short ImapCheckTimeout;
+WHERE char *ImapHomeNamespace INITVAL (NULL);
 #endif
 WHERE char *InReplyTo;
 WHERE char *Inbox;
@@ -52,6 +60,12 @@ WHERE char *Locale;
 WHERE char *MailcapPath;
 WHERE char *Maildir;
 WHERE char *MsgFmt;
+
+#ifdef MIXMASTER
+WHERE char *Mixmaster;
+WHERE char *MixEntryFormat;
+#endif
+
 WHERE char *Muttrc INITVAL (NULL);
 WHERE char *Outbox;
 WHERE char *Pager;
@@ -68,6 +82,7 @@ WHERE char *Prefix;
 WHERE char *PrintCmd;
 WHERE char *QueryCmd;
 WHERE char *Realname;
+WHERE char *SendCharset;
 WHERE char *Sendmail;
 WHERE char *Shell;
 WHERE char *Signature;
@@ -88,6 +103,7 @@ WHERE LIST *HeaderOrderList INITVAL(0);
 WHERE LIST *Ignore INITVAL(0);
 WHERE LIST *UnIgnore INITVAL(0);
 WHERE LIST *MailLists INITVAL(0);
+WHERE LIST *SubscribedLists INITVAL(0);
 
 /* bit vector for boolean variables */
 #ifdef MAIN_C
@@ -97,7 +113,11 @@ extern unsigned char Options[];
 #endif
 
 /* bit vector for the yes/no/ask variable type */
-WHERE unsigned long QuadOptions INITVAL (0);
+#ifdef MAIN_C
+unsigned char QuadOptions[(OPT_MAX*2 + 7) / 8];
+#else
+extern unsigned char QuadOptions[];
+#endif
 
 WHERE unsigned short Counter INITVAL (0);
 
@@ -109,6 +129,9 @@ WHERE short ReadInc;
 WHERE short SendmailWait;
 WHERE short Timeout;
 WHERE short WriteInc;
+WHERE short ScoreThresholdDelete;
+WHERE short ScoreThresholdRead;
+WHERE short ScoreThresholdFlag;
 
 /* flags for received signals */
 WHERE volatile sig_atomic_t SigAlrm INITVAL (0);
