@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -38,6 +38,9 @@
 #include "pgp.h"
 #endif
 
+#ifdef MIXMASTER
+#include "remailer.h"
+#endif
 
 
 static void append_signature (FILE *f)
@@ -335,9 +338,9 @@ static int include_forward (CONTEXT *ctx, HEADER *cur, FILE *out)
   if (option (OPTFORWDECODE))
   {
     cmflags |= M_CM_DECODE | M_CM_CHARCONV;
-    if (option (OPTFORWWEEDHEADER))
+    if (option (OPTWEED))
     {
-      chflags |= CH_WEED;
+      chflags |= CH_WEED | CH_REORDER;
       cmflags |= M_CM_WEED;
     }
   }
@@ -352,8 +355,8 @@ static int include_forward (CONTEXT *ctx, HEADER *cur, FILE *out)
 static int include_reply (CONTEXT *ctx, HEADER *cur, FILE *out)
 {
   char buffer[STRING];
-  int flags = M_CM_PREFIX | M_CM_DECODE | M_CM_CHARCONV;
-
+  int cmflags = M_CM_PREFIX | M_CM_DECODE | M_CM_CHARCONV;
+  int chflags = CH_DECODE;
 
 #ifdef _PGPPATH
   if (cur->pgp)
@@ -366,8 +369,6 @@ static int include_reply (CONTEXT *ctx, HEADER *cur, FILE *out)
   }
 #endif /* _PGPPATH */
 
-
-
   if (Attribution)
   {
     mutt_make_string (buffer, sizeof (buffer), Attribution, ctx, cur);
@@ -375,9 +376,15 @@ static int include_reply (CONTEXT *ctx, HEADER *cur, FILE *out)
     fputc ('\n', out);
   }
   if (!option (OPTHEADER))
-    flags |= M_CM_NOHEADER;
+    cmflags |= M_CM_NOHEADER;
+  if (option (OPTWEED))
+  {
+    chflags |= CH_WEED;
+    cmflags |= M_CM_WEED;
+  }
+
   mutt_parse_mime_message (ctx, cur);
-  mutt_copy_message (out, ctx, cur, flags, CH_DECODE);
+  mutt_copy_message (out, ctx, cur, cmflags, chflags);
   if (PostIndentString)
   {
     mutt_make_string (buffer, sizeof (buffer), PostIndentString, ctx, cur);
@@ -394,8 +401,15 @@ static int default_to (ADDRESS **to, ENVELOPE *env, int group)
 
   if (group && env->mail_followup_to)
   {
-    rfc822_append (to, env->mail_followup_to);
-    return 0;
+    snprintf (prompt, sizeof (prompt), _("Follow-up to %s%s?"),
+	      env->mail_followup_to->mailbox,
+	      env->mail_followup_to->next ? "..." : "");
+
+    if (query_quadoption (OPT_MFUPTO, prompt) == M_YES)
+    {
+      rfc822_append (to, env->mail_followup_to);
+      return 0;
+    }
   }
 
   if (!option(OPTREPLYSELF) && mutt_addr_is_user (env->from))
@@ -631,6 +645,7 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
 
     if (i == M_YES)
     {
+      mutt_message _("Including quoted message...");
       if (!cur)
       {
 	for (i = 0; i < ctx->vcount; i++)
@@ -649,14 +664,17 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
       }
       else
 	include_reply (ctx, cur, tempfp);
+
     }
   }
   else if (flags & SENDFORWARD)
   {
-    if (query_quadoption (OPT_MIMEFWD, _("Forward MIME encapsulated?")))
+    if ((i = query_quadoption (OPT_MIMEFWD, _("Forward MIME encapsulated?"))) == M_YES)
     {
       BODY *last = msg->content;
 
+      mutt_message _("Preparing forwarded message...");
+      
       while (last && last->next)
 	last = last->next;
 
@@ -686,7 +704,7 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
 	}
       }
     }
-    else
+    else if (i != -1)
     {
       if (cur)
 	include_forward (ctx, cur, tempfp);
@@ -695,6 +713,8 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
 	  if (ctx->hdrs[ctx->v2r[i]]->tagged)
 	    include_forward (ctx, ctx->hdrs[ctx->v2r[i]], tempfp);
     }
+    else if (i == -1)
+      return -1;
   }
 
 
@@ -711,7 +731,7 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
   }
 #endif
 
-
+  mutt_clear_error ();
 
   return (0);
 }
@@ -721,7 +741,8 @@ void mutt_set_followup_to (ENVELOPE *e)
   ADDRESS *t = NULL;
 
   /* only generate the Mail-Followup-To if the user has requested it, and
-     it hasn't already been set */
+   * it hasn't already been set
+   */
   if (option (OPTFOLLOWUPTO) && !e->mail_followup_to)
   {
     if (mutt_is_list_recipient (0, e->to, e->cc))
@@ -770,20 +791,30 @@ static ADDRESS *set_reverse_name (ENVELOPE *env)
   return (tmp);
 }
 
-static ADDRESS *mutt_default_from (void)
+ADDRESS *mutt_default_from (void)
 {
-  ADDRESS *adr = rfc822_new_address ();
+  ADDRESS *adr;
   const char *fqdn = mutt_fqdn(1);
-  
-  /* don't set realname here, it will be set later */
 
-  if (option (OPTUSEDOMAIN))
+  /* 
+   * Note: We let $from override $realname here.  Is this the right
+   * thing to do? 
+   */
+
+  if (From)
+    adr = rfc822_cpy_adr_real (From);
+  else if (option (OPTUSEDOMAIN))
   {
+    adr = rfc822_new_address ();
     adr->mailbox = safe_malloc (mutt_strlen (Username) + mutt_strlen (fqdn) + 2);
     sprintf (adr->mailbox, "%s@%s", NONULL(Username), NONULL(fqdn));
   }
   else
+  {
+    adr = rfc822_new_address ();
     adr->mailbox = safe_strdup (NONULL(Username));
+  }
+  
   return (adr);
 }
 
@@ -792,13 +823,19 @@ static int send_message (HEADER *msg)
   char tempfile[_POSIX_PATH_MAX];
   FILE *tempfp;
   int i;
-
+  
   /* Write out the message in MIME form. */
   mutt_mktemp (tempfile);
   if ((tempfp = safe_fopen (tempfile, "w")) == NULL)
     return (-1);
 
-  mutt_write_rfc822_header (tempfp, msg->env, msg->content, 0);
+#ifdef MIXMASTER
+  mutt_write_rfc822_header (tempfp, msg->env, msg->content, 0, msg->chain ? 1 : 0);
+#endif
+#ifndef MIXMASTER
+  mutt_write_rfc822_header (tempfp, msg->env, msg->content, 0, 0);
+#endif
+  
   fputc ('\n', tempfp); /* tie off the header. */
 
   if ((mutt_write_mime_body (msg->content, tempfp) == -1))
@@ -815,6 +852,11 @@ static int send_message (HEADER *msg)
     return (-1);
   }
 
+#ifdef MIXMASTER
+  if (msg->chain)
+    return mix_send_message (msg->chain, tempfile);
+#endif
+
   i = mutt_invoke_sendmail (msg->env->to, msg->env->cc, msg->env->bcc,
 		       tempfile, (msg->content->encoding == ENC8BIT));
   return (i);
@@ -851,8 +893,10 @@ ci_send_message (int flags,		/* send mode */
   FILE *tempfp = NULL;
   BODY *pbody;
   int i, killfrom = 0;
+
 #ifdef _PGPPATH
   BODY *save_content = NULL;
+  BODY *clear_content = NULL;
   char *pgpkeylist = NULL;
   /* save current value of "pgp_sign_as" */
   char *signas = NULL;
@@ -890,9 +934,9 @@ ci_send_message (int flags,		/* send mode */
   {
     msg = mutt_new_header ();
 
-    if (flags == SENDEDITMSG)
+    if (flags == SENDRESEND)
     {
-      if (mutt_prepare_edit_message(ctx, msg, cur) < 0)
+      if (mutt_prepare_template (ctx, msg, cur, 1) < 0)
 	goto cleanup;
     }
     else if (flags == SENDPOSTPONED)
@@ -901,7 +945,7 @@ ci_send_message (int flags,		/* send mode */
 	goto cleanup;
     }
 
-    if (flags & (SENDPOSTPONED | SENDEDITMSG))
+    if (flags & (SENDPOSTPONED|SENDRESEND))
     {
       if ((tempfp = safe_fopen (msg->content->filename, "a+")) == NULL)
       {
@@ -914,7 +958,7 @@ ci_send_message (int flags,		/* send mode */
       msg->env = mutt_new_envelope ();
   }
 
-  if (! (flags & (SENDKEY | SENDPOSTPONED | SENDEDITMSG)))
+  if (! (flags & (SENDKEY | SENDPOSTPONED | SENDRESEND)))
   {
     pbody = mutt_new_body ();
     pbody->next = msg->content; /* don't kill command-line attachments */
@@ -946,7 +990,7 @@ ci_send_message (int flags,		/* send mode */
   }
 
   /* this is handled here so that the user can match ~f in send-hook */
-  if (cur && option (OPTREVNAME) && !(flags & (SENDPOSTPONED | SENDEDITMSG)))
+  if (cur && option (OPTREVNAME) && !(flags & (SENDPOSTPONED|SENDRESEND)))
   {
     /* we shouldn't have to worry about freeing `msg->env->from' before
      * setting it here since this code will only execute when doing some
@@ -955,7 +999,7 @@ ci_send_message (int flags,		/* send mode */
     msg->env->from = set_reverse_name (cur->env);
   }
 
-  if (!msg->env->from && option (OPTUSEFROM) && !(flags & (SENDEDITMSG|SENDPOSTPONED)))
+  if (!msg->env->from && option (OPTUSEFROM) && !(flags & (SENDPOSTPONED|SENDRESEND)))
     msg->env->from = mutt_default_from ();
 
   if (flags & SENDBATCH) 
@@ -967,7 +1011,7 @@ ci_send_message (int flags,		/* send mode */
       process_user_header (msg->env);
     }
   }
-  else if (! (flags & (SENDPOSTPONED | SENDEDITMSG)))
+  else if (! (flags & (SENDPOSTPONED|SENDRESEND)))
   {
     if ((flags & (SENDREPLY | SENDFORWARD)) &&
 	envelope_defaults (msg->env, ctx, cur, flags) == -1)
@@ -1021,6 +1065,8 @@ ci_send_message (int flags,		/* send mode */
 	msg->pgp |= PGPENCRYPT;
       if (option (OPTPGPREPLYSIGN) && cur && cur->pgp & PGPSIGN)
 	msg->pgp |= PGPSIGN;
+      if (option (OPTPGPREPLYSIGNENCRYPTED) && cur && cur->pgp & PGPENCRYPT)
+	msg->pgp |= PGPSIGN;
     }
 #endif /* _PGPPATH */
 
@@ -1035,7 +1081,7 @@ ci_send_message (int flags,		/* send mode */
   }
   /* wait until now to set the real name portion of our return address so
      that $realname can be set in a send-hook */
-  if (msg->env->from && !msg->env->from->personal && !(flags & (SENDEDITMSG | SENDPOSTPONED)))
+  if (msg->env->from && !msg->env->from->personal && !(flags & (SENDRESEND|SENDPOSTPONED)))
     msg->env->from->personal = safe_strdup (Realname);
 
 
@@ -1090,7 +1136,7 @@ ci_send_message (int flags,		/* send mode */
 	mutt_edit_file (Editor, msg->content->filename);
     }
 
-    if (! (flags & (SENDPOSTPONED | SENDEDITMSG | SENDFORWARD | SENDKEY)))
+    if (! (flags & (SENDPOSTPONED | SENDFORWARD | SENDKEY | SENDRESEND)))
     {
       if (stat (msg->content->filename, &st) == 0)
       {
@@ -1110,7 +1156,7 @@ ci_send_message (int flags,		/* send mode */
   /* specify a default fcc.  if we are in batchmode, only save a copy of
    * the message if the value of $copy is yes or ask-yes */
 
-  if (!fcc[0] && (!(flags & SENDBATCH) || (quadoption (OPT_COPY) & 0x1)))
+  if (!fcc[0] && !(flags & (SENDRESEND|SENDPOSTPONED)) && (!(flags & SENDBATCH) || (quadoption (OPT_COPY) & 0x1)))
   {
     /* set the default FCC */
     if (!msg->env->from)
@@ -1133,6 +1179,7 @@ ci_send_message (int flags,		/* send mode */
   {
 main_loop:
 
+    mutt_pretty_mailbox (fcc);
     i = mutt_compose_menu (msg, fcc, sizeof (fcc), cur);
     if (i == -1)
     {
@@ -1192,36 +1239,35 @@ main_loop:
    * one place in order to avoid going to main_loop with encoded "env"
    * in case of error.  Ugh.
    */
+
 #ifdef _PGPPATH
   if (msg->pgp)
   {
     /* save the decrypted attachments */
-    save_content = msg->content;
+    clear_content = msg->content;
 
     if ((pgp_get_keys (msg, &pgpkeylist) == -1) ||
 	(pgp_protect (msg, pgpkeylist) == -1))
     {
-      if (msg->content->parts)
-      {
-	/* remove the toplevel multipart structure */
-	pbody = msg->content;
-	msg->content = msg->content->parts;
-	pbody->parts = NULL;
-	mutt_free_body (&pbody);
-      }
+      msg->content = mutt_remove_multipart (msg->content);
+      
       if (pgpkeylist)
 	FREE (&pgpkeylist);
+
       goto main_loop;
     }
   }
-#endif /* _PGPPATH */
 
-  if (flags & SENDEDITMSG)
-  {
-   int really_send = mutt_yesorno (_("Message edited. Really send?"), 1);
-   if (really_send != M_YES)
-     goto main_loop;
-  }
+  /* 
+   * at this point, msg->content is one of the following three things:
+   * - multipart/signed.  In this case, clear_content is a child.
+   * - multipart/encrypted.  In this case, clear_content exists
+   *   independently
+   * - application/pgp.  In this case, clear_content exists independently.
+   * - something else.  In this case, it's the same as clear_content.
+   */
+
+#endif /* _PGPPATH */
 
   if (!option (OPTNOCURSES) && !(flags & SENDMAILX))
     mutt_message _("Sending message...");
@@ -1230,6 +1276,7 @@ main_loop:
   encode_descriptions (msg->content);
 
   /* save a copy of the message, if necessary. */
+
   mutt_expand_path (fcc, sizeof (fcc));
   if (*fcc && mutt_strcmp ("/dev/null", fcc) != 0)
   {
@@ -1246,17 +1293,17 @@ main_loop:
       if (mutt_strcmp (msg->content->subtype, "encrypted") == 0 ||
 	  mutt_strcmp (msg->content->subtype, "signed") == 0)
       {
-	if (save_content->type == TYPEMULTIPART)
+	if (clear_content->type == TYPEMULTIPART)
 	{
 	  if (!(msg->pgp & PGPENCRYPT) && (msg->pgp & PGPSIGN))
 	  {
 	    /* save initial signature and attachments */
 	    save_sig = msg->content->parts->next;
-	    save_parts = msg->content->parts->parts->next;
+	    save_parts = clear_content->parts->next;
 	  }
 
 	  /* this means writing only the main part */
-	  msg->content = save_content->parts;
+	  msg->content = clear_content->parts;
 
 	  if (pgp_protect (msg, pgpkeylist) == -1)
 	  {
@@ -1268,17 +1315,7 @@ main_loop:
 	    goto full_fcc;
 	  }
 
-	  if (msg->pgp & PGPENCRYPT)
-	  {
-	    /* not released in pgp_encrypt_message() */
-	    mutt_free_body (&save_content->parts);
-	    /* make sure we release the right thing later */
-	    save_content->parts = msg->content;
-
-	    encode_descriptions (msg->content);
-	  }
-	  else
-	    save_content = msg->content;
+	  save_content = msg->content;
 	}
       }
       else
@@ -1291,6 +1328,7 @@ full_fcc:
 #endif /* _PGPPATH */
     if (msg->content)
       mutt_write_fcc (fcc, msg, NULL, 0, NULL);
+
     msg->content = tmpbody;
 
 #ifdef _PGPPATH
@@ -1305,22 +1343,33 @@ full_fcc:
       msg->content->parts->next = save_sig;
       msg->content->parts->parts->next = save_parts;
     }
+    else if (save_content)
+    {
+      /* destroy the new encrypted body. */
+      mutt_free_body (&save_content);
+    }
+      
 #endif /* _PGPPATH */
   }
 
-#ifdef _PGPPATH
-  if (msg->pgp & PGPENCRYPT)
-  {
-    /* cleanup structures from the first encryption */
-    mutt_free_body (&save_content);
-    FREE (&pgpkeylist);
-  }
-#endif /* _PGPPATH */
 
   if ((i = send_message (msg)) == -1)
   {
     if (!(flags & SENDBATCH))
     {
+#ifdef _PGPPATH
+      if ((msg->pgp & PGPENCRYPT) || 
+	  ((msg->pgp & PGPSIGN) && msg->content->type == TYPEAPPLICATION))
+      {
+	mutt_free_body (&msg->content); /* destroy PGP data */
+	msg->content = clear_content;	/* restore clear text. */
+      }
+      else if ((msg->pgp & PGPSIGN) && msg->content->type == TYPEMULTIPART)
+      {
+	mutt_free_body (&msg->content->parts->next);		/* destroy sig */
+	msg->content = mutt_remove_multipart (msg->content);	/* remove multipart */
+      }
+#endif
       msg->content = mutt_remove_multipart (msg->content);
       goto main_loop;
     }
@@ -1333,6 +1382,15 @@ full_fcc:
   else if (!option (OPTNOCURSES) && ! (flags & SENDMAILX))
     mutt_message (i == 0 ? _("Mail sent.") : _("Sending in background."));
 
+#ifdef _PGPPATH
+  if (msg->pgp & PGPENCRYPT)
+  {
+    /* cleanup structures from the first encryption */
+    mutt_free_body (&clear_content);
+    FREE (&pgpkeylist);
+  }
+#endif /* _PGPPATH */
+
   if (flags & SENDREPLY)
   {
     if (cur)
